"""
    Memory Asset
"""

from starfish.asset.asset_base import AssetBase

class MemoryAsset(AssetBase):
    """

    Memory asset can be used to try out assets without accessing any file system

    :param data: data string or byte text to save as the asset
    :type data: str or byte array
    :param metadata: Optional dictionary metadata to provide for the asset
        if non used then the class will generate a default metadata based on the data provided
    :type metadata: None or dict
    :param did: Optional did of the asset if it's registered
    :type did: None or str

    """
    def __init__(self, metadata=None, did=None, data=None):
        if metadata is None:
            metadata = {
                'name': 'MemoryAsset',
                'type': 'data',
                'author': 'memory asset'
            }
            if isinstance(data, str):
                metadata['contentType'] = 'text/plain; charset=utf-8'
            else:
                metadata['contentType'] = 'application/octet-stream'
        AssetBase.__init__(self, metadata, did)
        self._data = data

    @property
    def data(self):
        """

        Return the asset data

        :return: the asset data
        :type: str or byte
        """
        return self._data



import pytest
import pathlib
import json

METADATA_SAMPLE_PATH = pathlib.Path.cwd() / 'tests' / 'resources' / 'metadata' / 'sample_metadata1.json'

@pytest.fixture(scope="module")
def metadata():
    # load in the sample metadata
    assert METADATA_SAMPLE_PATH.exists(), "{} does not exist!".format(METADATA_SAMPLE_PATH)
    metadata = None
    with open(METADATA_SAMPLE_PATH, 'r') as file_handle:
        metadata = json.load(file_handle)

    return metadata



"""

Account class to provide basic functionality for all Ocean Accounts

"""

from starfish.models.squid_model import SquidModel
from web3 import Web3

class Account():
    """

    Account class, adds functionality for an account to be used by the Ocean network.

    At the moment the Account object is created by the :class:`Ocean` class.

    :param ocean: Ocean object
    :type ocean: :class:`.Ocean`
    :param address: address or dict of the account details
    :type address: string or dict
    :param password: password for the account
    :type password: str or None

    If the address parameter is a string then it's the account address.
    If dict then the dict can be the following format: ::

        {
            'address': 'xxxx',
            'password': 'yyyy',
        }

    """

    def __init__(self, ocean, address, password=None):
        """init a standard ocean agent"""
        self._ocean = ocean
        self._address = None
        self._password = None
        self._unlock_squid_account = None

        if isinstance(address, dict):
            self._address = address.get('address')
            self._password = address.get('password')
        elif isinstance(address, (tuple, list)):
            self._address = address[0]
            if len(address) > 1:
                self._password = address[1]
        elif isinstance(address, str):
            self._address = address
            self._password = password

#        if isinstance(self._address, str):
#            self._address = add_0x_prefix(self._address)

    def unlock(self, password=None):
        """

        Unlock the account so that it can be used to spend tokens/gas

        :param password: optional password to use to unlock this account, if none provided then the original password will be used.

        :type password: str or None

        >>> account.unlock('secret')
        True

        """
        if password is None:
            password = self._password

        if password is None:
            raise ValueError('You must provide an account password to unlock')

        # clear out the onlocked account for squid
        self._unlock_squid_account = None
        self._unlock_squid_account = self._squid_account
        if self._unlock_squid_account:
            self._unlock_squid_account.password = password
            return True
        return False

    def lock(self):
        """

        Lock the account, to stop access to this account for ethereum token transfer

        :return: True if this account was unlocked, else False if the account was not locked.
        :type: boolean

        >>> account.lock()
        True
        """
        if self._unlock_squid_account:
            self._unlock_squid_account = None
            return True
        return False

    def request_tokens(self, amount):
        """

        For **Testing Only**

        Request some ocean tokens to be transfere to this account address

        :param number amount: The amount of ocean tokens to transfer ( *Money for nothing* )

        :return: number of tokens transfered
        :type: int

        >>> account.request_tokens(100)
        100
        """
        model = self._ocean.get_squid_model()
        if model:
            if not self._unlock_squid_account:
                raise ValueError('You must unlock the account before requesting tokens')

            return model.request_tokens(self._unlock_squid_account, amount)
        return 0

    def is_address_equal(self, address):
        """

        Compares two addresses if are equal. Both addresses are converted to checksum
        address and then compared.

        :param str address: address to compare with this object's address
        :return: True if the param address is the same is the one held in this account
        :type: boolean

        >>> account = ocean.get_account('0x00bd138abd70e2f00903268f3db08f2d25677c9e')
        >>> account.is_address_equal('0x00Bd138aBD70e2F00903268F3Db08f2D25677C9e')
        True
        """
        return self.as_checksum_address == Web3.toChecksumAddress(address)

    @property
    def ocean(self):
        """

        Return the :class:`.Ocean` object for this account
        :return: The main ocean class
        :type: :class:`.Ocean`

        """
        return self._ocean

    @property
    def is_valid(self):
        """

        Return True if this account is registered in the Ocean network
        :return: True if this account address is valid
        :type: boolean

        >>> account.is_valid
        True
        """
        squid_account = self._squid_account
        return not squid_account is None

    @property
    def is_password(self):
        """
        Return True if the password has been set, else return False

        """
        return not self._password is None

    @property
    def _squid_account(self):
        """

        Return the squid account object used for squid services
        :return: The squid account object
        :type: object

        """

        if self._unlock_squid_account:
            return self._unlock_squid_account

        model = self._ocean.get_squid_model()
        if model:
            return model.get_account(self.as_checksum_address, self._password)
        return None

    @property
    def address(self):
        """

        Return the account address for this account

        :return: address
        :type: str

        >>> account = ocean.get_account('0x00bd138abd70e2f00903268f3db08f2d25677c9e')
        >>> account.address
        0x00bd138abd70e2f00903268f3db08f2d25677c9e
        """
        return self._address

    @property
    def as_checksum_address(self):
        """

        Return the address as a checksum address

        :return: checksum address
        :type: str

        >>> account = ocean.get_account('0x00bd138abd70e2f00903268f3db08f2d25677c9e')
        >>> account.as_checksum_address
        0x00Bd138aBD70e2F00903268F3Db08f2D25677C9e

        """
        if self._address:
            return Web3.toChecksumAddress(self._address.lower())
        return None

    @property
    def password(self):
        """

        Return the account password for this account

        :return: password
        :type: str

        >>> account.password
        secret

        """
        return self._password

    def set_password(self, password):
        """

        Set the password for this account

        :param str password: Password to set for this account

        >>> account.set_password('new secret')
        >>> account.password
        new secret
        """
        self.lock()
        self._password = password

    @property
    def ocean_balance(self):
        """

        Get the number of ocean tokens

        :return: number of ocean tokens
        :type: number

        >>> account.ocean_balance
        101

        """
        model = self._ocean.get_squid_model()
        if model:
            squid_account = self._squid_account
            if squid_account:
                balance = model.get_account_balance(squid_account)
                if balance:
                    return Web3.fromWei(balance.ocn, 'ether')
        return 0

    @property
    def ether_balance(self):
        """

        Get the number of ocean tokens

        :return: number of ocean tokens
        :type: number

        >>> account.ether_balance
        1000000001867769600000000000

        """
        model = self._ocean.get_squid_model()
        if model:
            squid_account = self._squid_account
            if squid_account:
                balance = model.get_account_balance(self._squid_account)
                if balance:
                    return Web3.fromWei(balance.eth, 'ether')
        return 0

    def __str__(self):
        return f'Account: {self.address}'



"""

Test asset modules in __init__.py

"""
import secrets
import logging
import json

from starfish.asset import (
    create_asset_from_metadata,
    AssetBase,
    MemoryAsset,
    SquidAsset,
    BundleAsset,
    OperationAsset,
    Asset,
)


def test_create_asset_from_metadata(metadata):
    asset = create_asset_from_metadata(metadata)
    assert(asset)
    assert(isinstance(asset, SquidAsset))

    type_list = [
        ( 'bundle', BundleAsset ), 
        ( 'data', MemoryAsset ),
        ( 'memory', MemoryAsset ),
        ( 'operation', OperationAsset ),
        ( 'unknown', Asset ),
    ]
    for type_name, class_type in type_list:
        metadata = {
            'type': type_name
        }
        asset = create_asset_from_metadata(metadata)
        assert(asset)
        assert(isinstance(asset, class_type))



"""

Agent class to provide basic functionality for all Ocean Agents

"""



import logging
from starfish.models.squid_model import SquidModel
from starfish.account import Account
from starfish.agent import AgentBase
from starfish.listing import Listing
from starfish.asset import (
    SquidAsset,
    Asset,
 )
from starfish.purchase import Purchase
from starfish.exceptions import StarfishPurchaseError
from starfish.models.squid_model import SquidModelPurchaseError
from starfish.operation.squid_operation import SquidOperation
from starfish.utils.did import did_parse
from squid_py.brizo.brizo_provider import BrizoProvider

import sys,traceback
logger = logging.getLogger('ocean')

class SquidAgent(AgentBase):
    """

    Squid Agent class allows to register and list asset listings.

    :param ocean: Ocean object that is being used.
    :type ocean: :class:`.Ocean`
    :param aquarius_url: Aquarius url ( http://localhost:5000 ).
    :type aquarius_url: str or None
    :param brizo_url: Brizo url (http://localhost:8030).
    :type brizo_url: str or None
    :param secret_store_url: Secret store URL (http://localhost:8010).
    :type secret_store_url: str or None
    :param parity_url: Parity URL, if you are using the secret store (http://localhost:9545').
    :type parity_url: str or None
    :param storage_path: Path to the storage db (squid_py.db).
    :type storage_path: str or None

    Example how to use this agent: ::

        # First import the classes
        from starfish.agent import SquidAgent
        from starfish import Ocean

        # create the ocean object
        ocean = Ocean()

        # get your publisher account
        account = ocean.get_account('0x00bd138abd70e2f00903268f3db08f2d25677c9e')

        #create the SquidAgent
        my_config = {
            'aquarius_url': 'http://localhost:5000',
            'brizo_url': 'http://localhost:8030',
            'secret_store_url': 'http://localhost:12001',
            'parity_url': 'http://localhost:9545',
            'storage_path': 'squid_py.db',
        }
        agent = SquidAgent(ocean, my_config)

        # register an asset data and listing info
        listing = agent.register(metadata, account)
    """

    def __init__(self, ocean, *args, **kwargs):
        """init a standard ocean object"""
        AgentBase.__init__(self, ocean)
        self._model = None

        if args and isinstance(args[0], dict):
            kwargs = args[0]

        self._aquarius_url = kwargs.get('aquarius_url', 'http://localhost:5000')
        self._brizo_url = kwargs.get('brizo_url', 'http://localhost:8030')
        self._secret_store_url = kwargs.get('secret_store_url', 'http://localhost:12001')
        self._storage_path = kwargs.get('storage_path', 'squid_py.db')

    def register_asset(self, asset, account):
        """

        Register a squid asset with the ocean network.

        :param asset: the SquidAsset to register, at the moment only a SquidAsset can be used.
        :type asset: :class:`.SquidAsset` object to register
        :param account: Ocean account to use to register this asset.
        :type account: :class:`.Account` object to use for registration.

        :return: A new :class:`.Listing` object that has been registered, if failure then return None.
        :type: :class:`.Listing` class

        For example::

            metadata = json.loads('my_metadata')
            # get your publisher account
            account = ocean.get_account('0x00bd138abd70e2f00903268f3db08f2d25677c9e')
            agent = SquidAgent(ocean)
            asset = SquidAsset(metadata)
            listing = agent.register_asset(asset, account)

            if listing:
                print(f'registered my listing asset for sale with the did {listing.did}')

        """

        if not isinstance(account, Account):
            raise TypeError('You need to pass an Account object')

        if not account.is_valid:
            raise ValueError('You must pass a valid account')

        if not account.is_password:
            raise ValueError('You must set the account password')

        model = self.squid_model

        ddo = model.register_asset(asset.metadata, account._squid_account)

        listing = None
        if ddo:
            asset.set_did(ddo.did)
            listing = Listing(self, ddo.did, asset, ddo)

        return listing


    def validate_asset(self, asset):
        """

        Validate an asset

        :param asset: Asset to validate.
        :return: True if the asset is valid
        """
        model = self.squid_model

        if not asset:
            raise ValueError('asset must be an object')
        if not isinstance(asset, Asset):
            raise ValueErrer('asset must be a type of Asset object')
        if not asset.metadata:
            raise ValueError('Metadata must have a value')
        if not isinstance(asset.metadata, dict):
            raise ValueError('Metadat must be a dict')

        return model.validate_metadata(asset.metadata)

    def get_listing(self, listing_id):
        """
        Return an listing from the given listing_id.

        :param str listing_id: Id of the listing.

        :return: a registered listing given a Id of the listing
        :type: :class:`.Listing` class

        """
        listing = None
        model = self.squid_model

        ddo = model.read_asset(listing_id)

        if ddo:
            asset = SquidAsset(ddo.metadata, ddo.did)
            listing = Listing(self, listing_id, asset, ddo)

        return listing


    def search_listings(self, text, sort=None, offset=100, page=1):
        """

        Search the off chain storage for an asset with the givien 'text'

        :param str text: Test to search all metadata items for.
        :param sort: sort the results ( defaults: None, no sort).
        :type sort: str or None
        :param int offset: Return the result from with the maximum record count ( defaults: 100 ).
        :param int page: Returns the page number based on the offset ( defaults: 1 ).

        :return: a list of assets objects found using the search.
        :type: list of DID strings

        For example::

            # return the 300 -> 399 records in the search for the text 'weather' in the metadata.
            my_result = agent.search_registered_assets('weather', None, 100, 3)

        """
        model = self.squid_model
        ddo_list = model.search_assets(text, sort, offset, page)
        return ddo_list

    def purchase_asset(self, listing, account):
        """

        Purchase an asset using it's listing and an account.

        :param listing: Listing to use for the purchase.
        :type listing: :class:`.Listing`
        :param account: Ocean account to purchase the asset.
        :type account: :class:`.Account` object to use for registration.

        """
        purchase = None
        model = self.squid_model

        ##avoid calling the default purchase (ocean.assets.order...), which uses callbacks
        ## to consume an asset
        metadata=listing.asset.metadata
        invoke_endpoint=metadata["base"].get("invoke_endpoint","asset")
        invokable=not(invoke_endpoint == "asset")
        if invokable==True:
            service_agreement_id = model.purchase_operation(listing.data, account._squid_account)
            purchase = SquidOperation(self, listing, service_agreement_id)
        else:
            service_agreement_id = model.purchase_asset(listing.data, account._squid_account)
            purchase = Purchase(self, listing, service_agreement_id)

        return purchase

    def is_access_granted_for_asset(self, asset, purchase_id, account):
        """

        Check to see if the account and purchase_id have access to the assed data.


        :param asset: Asset to check for access.
        :type asset: :class:`.Asset` object
        :param str purchase_id: purchase id that was used to purchase the asset.
        :param account: Ocean account to purchase the asset.
        :type account: :class:`.Account` object to use for registration.

        :return: True if the asset can be accessed and consumed.
        :type: boolean
        """

        model = self.squid_model
        return model.is_access_granted_for_asset(asset.did, purchase_id, account._squid_account)


    def purchase_wait_for_completion(self, purchase_id, asset, account, timeoutSeconds):
        """

        Wait for completion of the purchase

        TODO: issues here...
        + No method as yet to pass back paramaters and values during the purchase process
        + We assume that the following templates below will always be used.


        :param integer timeoutSeconds: Optional seconds to waif to purchase to complete. Default: 60 seconds
        :return: True if successfull or an error message if failed
        :type: string or boolean

        :raises OceanPurchaseError: if the correct events are not received

        """
        model = self.squid_model
        if not purchase_id:
            raise ValueError('Please provide a valid purhase id')

        try:
            model.purchase_wait_for_completion(purchase_id, asset.did, account._squid_account, timeoutSeconds)
        except SquidModelPurchaseError as purchaseError:
            raise StarfishPurchaseError(purchaseError)
        except Exception as e:
            raise e
        return True


    def consume_asset(self, listing, purchase_id, account, download_path ):
        """
        Consume the asset and download the data. The actual payment to the asset
        provider will be made at this point.

        :param listing: Listing that was used to make the purchase.
        :type listing: :class:`.Listing`
        :param str purchase_id: purchase id that was used to purchase the asset.
        :param account: Ocean account that was used to purchase the asset.
        :type account: :class:`.Account` object to use for registration.
        :param str download_path: path to store the asset data.

        :return: True if the asset has been consumed and downloaded
        :type: boolean

        """
        model = self.squid_model
        return model.consume_asset(listing.data, purchase_id, account._squid_account, download_path)

    @staticmethod
    def invoke_operation(listing, purchase_id, account, payload):
        """
        Invoke the operation

        :param listing: Listing that was used to make the purchase.
        :type listing: :class:`.Listing`
        :param str purchase_id: purchase id that was used to purchase the asset.
        :param account: Ocean account that was used to purchase the asset.
        :type account: :class:`.Account` object to use for registration.
        :param str payload: params required for the operation

        :return: True if the operation was invoked
        :type: boolean

        """
        logger.info(f'calling invoke in squid_agent.py with payload: {payload}')
        try :
            ddo= listing.data
            files = ddo.metadata['base']['encryptedFiles']
            logger.info(f'encrypted contentUrls: {files}')
            files = files if isinstance(files, str) else files[0]
            sa = SquidModel.get_service_agreement_from_ddo(ddo)
            service_url = sa.service_endpoint
            if not service_url:
                logger.error(
                    'Consume asset failed, service definition is missing the "serviceEndpoint".')
                raise AssertionError(
                    'Consume asset failed, service definition is missing the "serviceEndpoint".')

            return BrizoProvider.get_brizo().invoke_service(
                purchase_id, service_url,
                account.address, payload)
        except Exception:
            logger.error('got error invoking Brizo')
            traceback.print_exc(file=sys.stdout)
            return False



    @property
    def squid_model(self):
        """

        Return an instance of the squid model, for access to the squid library layer
        :return: squid model object
        """

        if not self._model:
            options = {
                'aquarius_url': self._aquarius_url,
                'brizo_url': self._brizo_url,
                'secret_store_url': self._secret_store_url,
                'storage_path': self._storage_path,
            }
            self._model = self._ocean.get_squid_model(options)
        return self._model

    @staticmethod
    def is_did_valid(did):
        """
        Checks to see if the DID string is a valid DID for this type of Asset.
        This method only checks the syntax of the DID, it does not resolve the DID
        to see if it is assigned to a valid Asset.

        :param str did: DID string to check to see if it is in a valid format.

        :return: True if the DID is in the format 'did:op:xxxxx'
        :type: boolean
        """
        data = did_parse(did)
        return not data['path']



"""
    Test ocean class

"""

import pathlib
import json
import logging
import time
from web3 import Web3

from starfish import Ocean
from starfish.models.squid_model import SquidModel
from starfish.agent import SquidAgent
from starfish.asset import SquidAsset

from starfish.logging import setup_logging

from squid_py.agreements.service_factory import ServiceDescriptor
from squid_py.utils.utilities import generate_new_id
from squid_py.brizo.brizo_provider import BrizoProvider

from tests.integration.mocks.koi_mock import KoiMock
from tests.integration.mocks.brizo_mock import BrizoMock


def _register_asset_for_sale(agent, metadata, account):

    asset=SquidAsset(metadata)
    listing = agent.register_asset(asset, account=account)
    assert listing
    assert listing.asset.did
    return listing

def test_invoke(ocean, metadata, config, brizo_mock):


    agent = SquidAgent(ocean, config.squid_config)
    assert agent


    # test node has the account #0 unlocked
    publisher_account = ocean.get_account(config.publisher_account)
    publisher_account.unlock()
    publisher_account.request_tokens(20)

    listing = _register_asset_for_sale(agent, metadata, publisher_account)
    assert listing
    assert publisher_account

    listing_did = listing.asset.did
    # start to test getting the asset from storage
    listing = agent.get_listing(listing_did)
    assert listing
    assert listing.asset.did == listing_did


    purchase_account = ocean.get_account(config.purchaser_account)
    logging.info(f'purchase_account {purchase_account.ocean_balance}')

    purchase_account.unlock()

    purchase_account.request_tokens(10)

    time.sleep(1)
    logging.info(f'purchase_account after token request {purchase_account.ocean_balance}')

    model = ocean.get_squid_model()
    ddo = model._squid_ocean.assets.resolve(listing.asset.did)

    brizo_mock.subscribe(ocean, publisher_account._squid_account, listing.asset.did, ddo)


    # test purchase an asset
    # this purchase does not automatically fire a consume() request as no callback is registered
    purchase_asset = listing.purchase(purchase_account)
    assert purchase_asset

    assert(not purchase_asset.is_completed(purchase_account))

    error_message = purchase_asset.wait_for_completion(purchase_account)
    assert(error_message == True)

    assert(purchase_asset.is_completed(purchase_account))

    # assert len(os.listdir(consumer_ocean_instance.config.downloads_path)) == downloads_path_elements + 1

    # This test does not work with the current barge

    assert purchase_asset.is_purchased
    #assert purchase_asset.is_purchase_valid(purchase_account)
    #this assertion fails
    purch_type=purchase_asset.get_type
    logging.debug(f'purchase type {purch_type}')
    paramvalue={'hello':'world'}
    print(purchase_asset)
    # TODO: invoke method does not exist?
    # result=purchase_asset.invoke(purchase_account,{'operation':'echo','params':paramvalue})
    ## TBD: asset on the result of the invoke
    # logging.debug(f'invoke result {result}')
    # assert result == paramvalue


#test_asset()



"""
    test_19_asset_download

    As a developer working with an Ocean marketplace,
    I want to record a purchase.

"""

import secrets
import logging
import json

from starfish.asset import MemoryAsset


def test_19_asset_download(surfer_agent, metadata):
    test_data = secrets.token_hex(1024)
    asset = MemoryAsset(metadata=metadata, data=test_data)
    listing = surfer_agent.register_asset(asset)
    logging.debug("asset_download for listingid: " + listing.listing_id + " = asset_id: " +  asset.asset_id)
    asset2 = surfer_agent.download_asset(asset.asset_id)
    logging.debug("download_asset response: " + str(asset2))
    # FIXME SurferAgent.download_asset under development
    # assert(asset2.asset_id == asset.asset_id)
    # assert(asset2.data == asset.data)



"""
    Squid Asset
"""

from starfish.asset.asset_base import AssetBase

class SquidAsset(AssetBase):
    """

    Create a new squid asset to use in the Ocean network

    :param metadata: metadata to store for this asset, it must be a valid squid metadata dict
    :type metadata: dict
    :param did: did of the asset if it is registered, can be None for a new non registered asset
    :type did: None or str

    """

    @staticmethod
    def generate_metadata():
        """
        Return a generated example metadata that can be changed by the child asset class

        :return: default metadata
        :type: dict
        """
        metadata = {
            'name': "Ocean protocol white paper",
            'type': "dataset",
            'description': "Introduce the main concepts and vision behind ocean protocol",
            'size': "1mb",
            'dateCreated': "2012-10-10T17:00:000Z",
            'author': "Ocean Protocol Foundation Ltd.",
            'license': "CC-BY",
            'copyrightHolder': "Ocean Protocol Foundation Ltd.",
            'encoding': "UTF-8",
            'compression': "",
            'contentType': "text/csv",
            'workExample': "Text PDF",
            'inLanguage': "en",
            'categories': ["white-papers"],
            'tags': "data exchange sharing curation bonding curve",
            'price': 23,
            'files': [
                {
                    "url": "https://testocnfiles.blob.core.windows.net/testfiles/testzkp.pdf",
                    "checksum": "efb2c764274b745f5fc37f97c6b0e761",
                    "checksumType": "MD5",
                    "contentLength": "4535431",
                    "resourceId": "access-log2018-02-13-15-17-29-18386C502CAEA932"
                },
                {
                    "url": "s3://ocean-test-osmosis-data-plugin-dataseeding-1537375953/data.txt",
                    "checksum": "efb2c764274b745f5fc37f97c6b0e761",
                    "contentLength": "4535431",
                    "resourceId": "access-log2018-02-13-15-17-29-18386C502CAEA932"
                },
                {
                    "url": "http://www3.cs.stonybrook.edu/~algorith/implement/graphbase/distrib/cweb3"
                     ".4e.tar.gz"
                },
            ],
            'links': [
                {
                    "sample1": "http://data.ceda.ac.uk/badc/ukcp09/data/gridded-land-obs/gridded-land"
                    "-obs-daily/",
                },
                {
                    "sample2": "http://data.ceda.ac.uk/badc/ukcp09/data/gridded-land-obs/gridded-land"
                    "-obs-averages-25km/",
                },
                {
                    "fieldsDescription": "http://data.ceda.ac.uk/badc/ukcp09/",
                },
            ],
        }
        return metadata



""""

Test examples

"""


def test_register_asset():
    from examples.register_asset import main
    main()

def test_register_memory_asset():
    from examples.register_memory_asset import main
    main()


def test_surfer_memory_asset():
    from examples.surfer_memory_asset import main
    main()



"""
    Setup logging, this ideally must be called only once an the start of using
    this library.

"""

import os
import logging.config
import logging
import yaml
import coloredlogs


def setup_logging(level=logging.INFO, filename='logging.yaml', env_key='LOG_CFG'):
    """
    **@author:** Prathyush SP, but hacked around by Bill
    Logging Setup
    """
    value = os.getenv(env_key, None)
    if value:
        filename = value

    # start basic logging
    logging.basicConfig(level=level)
    coloredlogs.install(level=level)

    if os.path.exists(filename):
        with open(filename, 'rt') as file_handle:
            try:
                config = yaml.safe_load(file_handle.read())
                logging.config.dictConfig(config)
                logging.info(f'Logging configuration loaded from file: {filename}')
            except yaml.YAMLError as error_exception:
                print(error_exception)
                logging.error(f'Failed to load logging configuration file {filename}: {error_exception}')



from unittest.mock import Mock
import pytest
import secrets
import logging
import pathlib

from tests.integration.libs.integration_test_config import IntegrationTestConfig

from squid_py.brizo.brizo_provider import BrizoProvider
from squid_py.brizo.brizo import Brizo

from starfish import Ocean
from starfish.agent import SurferAgent
from starfish.models.surfer_model import SurferModel

from tests.integration.mocks.brizo_mock import BrizoMock

CONFIG_FILE_PATH = pathlib.Path.cwd() / 'tests' / 'integration' / 'config.ini'
INVOKE_CONFIG_FILE_PATH = pathlib.Path.cwd() / 'tests' / 'integration' / 'invoke_config.ini'


@pytest.fixture(scope="module")
def ocean():
    integrationTestConfig = IntegrationTestConfig(CONFIG_FILE_PATH)
    ocean = Ocean(
        keeper_url=integrationTestConfig.keeper_url,
        contracts_path=integrationTestConfig.contracts_path,
        gas_limit=integrationTestConfig.gas_limit,
        log_level=logging.WARNING
    )

    return ocean

@pytest.fixture(scope="module")
def brizo_mock():
    BrizoProvider.set_brizo_class(BrizoMock)
    mock = BrizoProvider.get_brizo()
    return mock

@pytest.fixture(scope="module")
def config():
    integrationTestConfig = IntegrationTestConfig(CONFIG_FILE_PATH)
    return integrationTestConfig


@pytest.fixture(scope="module")
def surfer_agent(ocean):
    integrationTestConfig = IntegrationTestConfig(CONFIG_FILE_PATH)

    ddo_options = None
    if integrationTestConfig.koi_url:
        ddo_options = {
            'invoke': f'{integrationTestConfig.koi_url}/api/v1',
        }
    ddo = SurferAgent.generate_ddo(integrationTestConfig.surfer_url, ddo_options)
    options = {
        'url': integrationTestConfig.surfer_url,
        'username': integrationTestConfig.surfer_username,
        'password': integrationTestConfig.surfer_password,
    }
    return SurferAgent(ocean, did=ddo.did, ddo=ddo, options=options)



"""
    test_07_metadata_access

    As a developer working with Ocean,
    I need a way to request metadata for an arbitrary asset

"""

import secrets
import json

from starfish.asset import MemoryAsset
from starfish.agent import SurferAgent
from starfish.ddo.starfish_ddo import StarfishDDO



def test_07_metadata_access(ocean, surfer_agent, metadata):
    test_data = secrets.token_hex(1024)
    asset = MemoryAsset(metadata=metadata, data=test_data)
    listing = surfer_agent.register_asset(asset)
    assert(not listing is None)
    assert(listing.listing_id)
    store_listing = surfer_agent.get_listing(listing.listing_id)
    assert(store_listing)
    assert(store_listing.asset.asset_id)
    assert(store_listing.asset.metadata)
    assert(store_listing.asset.metadata == listing.asset.metadata)
